@@ -317,8 +317,10 @@ arp_free(struct arp_state *astate)
 		eloop_timeout_delete(astate->iface->ctx->eloop, NULL, astate);
 		state = D_STATE(astate->iface);
 		TAILQ_REMOVE(&state->arp_states, astate, next);
-		if (state->arp_ipv4ll == astate)
+		if (state->arp_ipv4ll == astate) {
+			ipv4ll_stop(astate->iface);
 			state->arp_ipv4ll = NULL;
+		}
 		free(astate);
 	}
 }
@@ -352,6 +354,10 @@ arp_close(struct interface *ifp)
 	}
 
 	while ((astate = TAILQ_FIRST(&state->arp_states))) {
+#ifndef __clang_analyzer__
+		/* clang guard needed for a more compex variant on this bug:
+		 * http://llvm.org/bugs/show_bug.cgi?id=18222 */
 		arp_free(astate);
+#endif
 	}
 }
