@@ -37,16 +37,70 @@
 
 #include "config.h"
 
-#ifdef HAVE_VIS_H
-#include <vis.h>
-#endif
-
 #include "common.h"
 #include "dhcp-common.h"
 #include "dhcp.h"
 #include "if.h"
 #include "ipv6.h"
 
+void
+dhcp_print_option_encoding(const struct dhcp_opt *opt, int cols)
+{
+
+	while (cols < 40) {
+		putchar(' ');
+		cols++;
+	}
+	putchar('\t');
+	if (opt->type & EMBED)
+		printf(" embed");
+	if (opt->type & ENCAP)
+		printf(" encap");
+	if (opt->type & INDEX)
+		printf(" index");
+	if (opt->type & ARRAY)
+		printf(" array");
+	if (opt->type & UINT8)
+		printf(" byte");
+	else if (opt->type & UINT16)
+		printf(" uint16");
+	else if (opt->type & SINT16)
+		printf(" sint16");
+	else if (opt->type & UINT32)
+		printf(" uint32");
+	else if (opt->type & SINT32)
+		printf(" sint32");
+	else if (opt->type & ADDRIPV4)
+		printf(" ipaddress");
+	else if (opt->type & ADDRIPV6)
+		printf(" ip6address");
+	else if (opt->type & FLAG)
+		printf(" flag");
+	else if (opt->type & RFC3397)
+		printf(" domain");
+	else if (opt->type & DOMAIN)
+		printf(" dname");
+	else if (opt->type & ASCII)
+		printf(" ascii");
+	else if (opt->type & RAW)
+		printf(" raw");
+	else if (opt->type & BINHEX)
+		printf(" binhex");
+	else if (opt->type & STRING)	
+		printf(" string");
+	if (opt->type & RFC3361)	
+		printf(" rfc3361");
+	if (opt->type & RFC3442)
+		printf(" rfc3442");
+	if (opt->type & RFC5969)
+		printf(" rfc5969");
+	if (opt->type & REQUEST)
+		printf(" request");
+	if (opt->type & NOREQ)
+		printf(" norequest");
+	putchar('\n');
+}
+
 struct dhcp_opt *
 vivso_find(uint32_t iana_en, const void *arg)
 {
@@ -293,60 +347,179 @@ decode_rfc3397(char *out, size_t len, const uint8_t *p, size_t pl)
 			*out = '\0';
 	}
 
+	if (count)
+		/* Don't count the trailing NUL */
+		count--;
 	return (ssize_t)count;
 }
 
-/*
- * Escape these characters to avoid any nastiness passing to a POSIX shell.
- * See IEEE Std 1003.1, 2004 Shell Command Language, 2.2 Quoting
- * space is not escaped.
- */
-#define ESCAPE_CHARS	"|&;<>()$`\\\"'\t\n"
+/* Check for a valid domain name as per RFC1123 with the exception of
+ * allowing - and _ (but not at start or end) as they seem to be widely used. */
+static int
+valid_domainname(char *lbl, int type)
+{
+	char *slbl, *lst;
+	unsigned char c;
+	int start, len, errset;
+
+	if (lbl == NULL || *lbl == '\0') {
+		errno = EINVAL;
+		return 0;
+	}
+
+	slbl = lbl;
+	lst = NULL;
+	start = 1;
+	len = errset = 0;
+	for (;;) {
+		c = (unsigned char)*lbl++;
+		if (c == '\0')
+			return 1;
+		if (c == ' ') {
+			if (lbl - 1 == slbl) /* No space at start */
+				break;
+			if (!(type & ARRAY))
+				break;
+			/* Skip to the next label */
+			if (!start) {
+				start = 1;
+				lst = lbl - 1;
+			}
+			if (len)
+				len = 0;
+			continue;
+		}
+		if (c == '.') {
+			if (*lbl == '.')
+				break;
+			len = 0;
+			continue;
+		}
+		if (((c == '-' || c == '_') &&
+		    !start && *lbl != ' ' && *lbl != '\0') ||
+		    isalnum(c))
+		{
+			if (++len > 63) {
+				errno = ERANGE;
+				errset = 1;
+				break;
+			}
+		} else
+			break;
+		if (start)
+			start = 0;
+	}
+
+	if (!errset)
+		errno = EINVAL;
+	if (lst) {
+		/* At least one valid domain, return it */
+		*lst = '\0';
+		return 1;
+	}
+	return 0;
+}
 
 /*
  * Prints a chunk of data to a string.
- * Escapes some characters defnined above to try and avoid any loopholes
- * in the shell we're passing to.
- * Any non visible characters are escaped as an octal number.
+ * PS_SHELL goes as it is these days, it's upto the target to validate it.
+ * PS_SAFE has all non ascii and non printables changes to escaped octal.
  */
+static const char hexchrs[] = "0123456789abcdef";
 ssize_t
-print_string(char *s, size_t len, const uint8_t *data, size_t dl)
+print_string(char *dst, size_t len, int type, const uint8_t *data, size_t dl)
 {
+	char *odst;
 	uint8_t c;
-	const uint8_t *e, *p;
+	const uint8_t *e;
 	size_t bytes;
-	char v[5], *vp, *ve;
 
+	odst = dst;
 	bytes = 0;
 	e = data + dl;
+
 	while (data < e) {
 		c = *data++;
-		if (c == '\0') {
-			/* If rest is all NULL, skip it. */
-			for (p = data; p < e; p++)
-				if (*p != '\0')
-					break;
-			if (p == e)
-				break;
+		if (type & BINHEX) {
+			if (dst) {
+				if (len  == 0 || len == 1) {
+					errno = ENOSPC;
+					return -1;
+				}
+				*dst++ = hexchrs[(c & 0xF0) >> 4];
+				*dst++ = hexchrs[(c & 0x0F)];
+				len -= 2;
+			}
+			bytes += 2;
+			continue;
 		}
-		ve = svis(v, c, VIS_CSTYLE | VIS_OCTAL,
-		    data <= e ? *data : 0, ESCAPE_CHARS);
-		if (s && len < (size_t)(ve - v) + 1) {
-			errno = ENOBUFS;
-			return -1;
+		if (type & ASCII && (!isascii(c))) {
+			errno = EINVAL;
+			break;
 		}
-		bytes += (size_t)(ve - v);
-		if (s) {
-			vp = v;
-			while (vp != ve)
-				*s++ = *vp++;
+		if (!(type & (ASCII | RAW | ESCSTRING)) /*plain string */ &&
+		    (!isascii(c) && !isprint(c)))
+		{
+			errno = EINVAL;
+			break;
+		}
+		if (type & ESCSTRING &&
+		    (c == '\\' || !isascii(c) || !isprint(c)))
+		{
+			errno = EINVAL;
+			if (c == '\\') {
+				if (dst) {
+					if (len  == 0 || len == 1) {
+						errno = ENOSPC;
+						return -1;
+					}
+					*dst++ = '\\'; *dst++ = '\\';
+					len -= 2;
+				}
+				bytes += 2;
+				continue;
+			}
+			if (dst) {
+				if (len < 5) {
+					errno = ENOSPC;
+					return -1;
+				}
+				*dst++ = '\\';
+		                *dst++ = (((unsigned char)c >> 6) & 03) + '0';
+		                *dst++ = (((unsigned char)c >> 3) & 07) + '0';
+		                *dst++ = ( (unsigned char)c       & 07) + '0';
+				len -= 4;
+			}
+			bytes += 4;
+		} else {
+			if (dst) {
+				if (len == 0) {
+					errno = ENOSPC;
+					return -1;
+				}
+				*dst++ = (char)c;
+				len--;
+			}
+			bytes++;
 		}
 	}
 
 	/* NULL */
-	if (s)
-		*s = '\0';
-	bytes++;
+	if (dst) {
+		if (len == 0) {
+			errno = ENOSPC;
+			return -1;
+		}
+		*dst = '\0';
+
+		/* Now we've printed it, validate the domain */
+		if (type & DOMAIN && !valid_domainname(odst, type)) {
+			*odst = '\0';
+			return 1;
+		}
+
+	}
+
 	return (ssize_t)bytes;
 }
 
@@ -421,12 +594,12 @@ print_option(char *s, size_t len, int type, const uint8_t *data, size_t dl,
 		sl = decode_rfc3397(NULL, 0, data, dl);
 		if (sl == 0 || sl == -1)
 			return sl;
-		l = (size_t)sl;
+		l = (size_t)sl + 1;
 		tmp = malloc(l);
 		if (tmp == NULL)
 			return -1;
 		decode_rfc3397(tmp, l, data, dl);
-		sl = print_string(s, len, (uint8_t *)tmp, l - 1);
+		sl = print_string(s, len, type, (uint8_t *)tmp, l - 1);
 		free(tmp);
 		return sl;
 	}
@@ -436,7 +609,7 @@ print_option(char *s, size_t len, int type, const uint8_t *data, size_t dl,
 		if ((tmp = decode_rfc3361(data, dl)) == NULL)
 			return -1;
 		l = strlen(tmp);
-		sl = print_string(s, len, (uint8_t *)tmp, l);
+		sl = print_string(s, len, type, (uint8_t *)tmp, l);
 		free(tmp);
 		return sl;
 	}
@@ -448,19 +621,15 @@ print_option(char *s, size_t len, int type, const uint8_t *data, size_t dl,
 		return decode_rfc5969(s, len, data, dl);
 #endif
 
-	if (type & STRING) {
-		/* Some DHCP servers return NULL strings */
-		if (*data == '\0')
-			return 0;
-		return print_string(s, len, data, dl);
-	}
+	if (type & STRING)
+		return print_string(s, len, type, data, dl);
 
 	if (type & FLAG) {
 		if (s) {
 			*s++ = '1';
 			*s = '\0';
 		}
-		return 2;
+		return 1;
 	}
 
 	if (!s) {
@@ -494,22 +663,20 @@ print_option(char *s, size_t len, int type, const uint8_t *data, size_t dl,
 					l += (size_t)sl;
 				data += 16;
 			}
-			return (ssize_t)(l + 1);
+			return (ssize_t)l;
 		}
 #endif
-		else if (type & BINHEX) {
-			l = 2;
-		} else {
+		else {
 			errno = EINVAL;
 			return -1;
 		}
-		return (ssize_t)((l + 1) * dl);
+		return (ssize_t)(l * dl);
 	}
 
 	t = data;
 	e = data + dl;
 	while (data < e) {
-		if (data != t && type != BINHEX) {
+		if (data != t) {
 			*s++ = ' ';
 			bytes++;
 			len--;
@@ -554,10 +721,7 @@ print_option(char *s, size_t len, int type, const uint8_t *data, size_t dl,
 			data += 16;
 		}
 #endif
-		else if (type & BINHEX) {
-			sl = snprintf(s, len, "%.2x", data[0]);
-			data++;
-		} else
+		else
 			sl = 0;
 		len -= (size_t)sl;
 		bytes += sl;
@@ -587,7 +751,7 @@ dhcp_envoption1(char **env, const char *prefix,
 		e = 0;
 	if (prefix)
 		e += strlen(prefix);
-	e += (size_t)len + 4;
+	e += (size_t)len + 2;
 	if (env == NULL)
 		return e;
 	v = val = *env = malloc(e);
@@ -600,7 +764,7 @@ dhcp_envoption1(char **env, const char *prefix,
 	else
 		v += snprintf(val, e, "%s=", prefix);
 	if (len != 0)
-		print_option(v, (size_t)len, opt->type, od, ol, ifname);
+		print_option(v, (size_t)len + 1, opt->type, od, ol, ifname);
 	return e;
 }
 
